#!/bin/bash

DIR=$(dirname "$0")

if [ ! -x "$DIR/qannotate/qannotate" ]; then
	if [ ! -e "$DIR/qannotate/Makefile" ]; then
		echo "qannotate executable missing. Goto qannotate directory, run qmake and make"
	else
		echo "qannotate executable missing. Run make in this directory"
	fi
	exit 1
fi

if [ ! -x "$DIR/noaadata/noaadata" ]; then
	echo "noaadata executable missing. Run make in this directory"
	exit 1
fi

makevid() {
	rm "$DIR/noaa_air_2m_img"/*
	rm "$DIR/noaa_air_2m_png"/*

	NFRAMESTART=300
	NFRAMEEND=600

	NDAYS=$2
	#NDAYS=90
	#NDAYS=270
	#NDAYS=365

	YAVG=$1
	#YAVG=30
	#YAVG=73

	# Calculate average and daily difference into .ppm images
	"$DIR/noaadata/noaadata" -y $YAVG -z $NDAYS

	NIMG=$(ls "$DIR/noaa_air_2m_img/" | wc -l)

	#- Annotate the result and scale images
	"$DIR/qannotate/qannotate" -s "$DIR/noaa_air_2m_img" -d "$DIR/noaa_air_2m_png" -n $NIMG -o $NFRAMESTART -x $YAVG -z $NDAYS

	#- Run first slide text
	convert "$DIR/noaa_air_2m_png/$(printf '%05d' $NFRAMESTART).ppm.png" \
		-fill '#ffffff80' \
		-draw 'rectangle 0,0 768,433' \
		-fill 'black' \
		-font DejaVu-Sans-Bold -pointsize 40 -gravity center \
		-annotate +0-135 'NOAA NCEP/NCAR data' \
		-annotate +0-87 '2m temperature reanalysis' \
		-annotate +0-39 'Daily difference between' \
		-annotate +0+9 "$NDAYS day moving average and" \
		-annotate +0+57 "$YAVG year average [1948..$((1948 + $YAVG - 1))]" \
		-font DejaVu-Sans -pointsize 14 \
		-annotate +0+95 'Reanalysis data provided by NOAA/OAR/ESRL PSL Boulder Colorado USA. Dataset source reference:' \
		-font DejaVu-Sans-Oblique -pointsize 14 \
		-annotate +0+113 'Kalnay et al., The NCEP/NCAR 40-year reanalysis project, Bull. Amer. Meteor. Soc., 77, 437-470, 1996' \
		-font DejaVu-Sans -pointsize 14 \
		-annotate +0+131 'See: https://psl.noaa.gov/data/gridded/data.ncep.reanalysis.html' \
		-annotate +0+149 'Files: daily averages: air.2m.gauss.*.nc' \
		"$DIR/first.text.png"

	# Copy first slide into start frames
	for i in $(seq -f'%05.0f' 0 $(($NFRAMESTART - 1)))
	do
		cp "$DIR/first.text.png" "$DIR/noaa_air_2m_png/$i.ppm.png"
	done

	rm "$DIR/first.text.png"

	#- Make a tail og same frames
	for i in $(seq -f'%05.0f' $(($NIMG + $NFRAMESTART)) $(($NIMG + $NFRAMESTART + $NFRAMEEND - 1)))
	do
		cp "$DIR/noaa_air_2m_png/$(($NIMG + $NFRAMESTART - 1)).ppm.png" "$DIR/noaa_air_2m_png/$i.ppm.png"
	done

	#- Create a video
	#ffmpeg -loglevel warning -y -f image2 -framerate 60 -i $DIR/noaa_air_2m_png/%05d.ppm.png noaa_air_2m_gauss.mkv
	ffmpeg -loglevel warning -y -f image2 -framerate 60 \
		-i "$DIR/noaa_air_2m_png/%05d.ppm.png" \
		"$DIR/noaa_air_2m_gauss-daily_diff-$NDAYS-avg_1948-$((1948 + $YAVG - 1)).mp4"
}

makevid 30 365
makevid 73 365
makevid 30 270
makevid 30 90
