#!/bin/bash

DIR=$(dirname "$0")

BORDER="10"
LASTYEAR="2021"
AVGSTARTYEAR="1948"

IMGBASENAME="noaa_air_2m_gauss-daily_diff-avg_1948-1977_latitude_avg"

$DIR/noaadataline -s $AVGSTARTYEAR -i "$DIR/../noaa_air_2m" > "$DIR/lines.ppm" 2> "$DIR/minmax.txt"

NDAYS="$(head -2 "$DIR/lines.ppm" | tail -1 | cut -d' ' -f2)"
MINMAX=$(tail -1 "$DIR/minmax.txt" | sed -e's, ,/,')

for i in $(seq 0 1461 $NDAYS); do
	convert "$DIR/lines.ppm" -crop 94x1461+0+$i -rotate -90 -flip "$DIR/part_$(printf '%05d' $i).png"
done

montage "$DIR/part_*.png" -tile 1x -geometry +0+16 -background black "$DIR/mont.png"
convert "$DIR/mont.png" -bordercolor black -border $BORDER "$DIR/border.png"

# construct a draw command for convert
# make divisions and put in year numbers
A=""
YEAR=1949
donedays=0
YMSEQA="31 28 31 30 31 30 31 31 30 31 30"
YMSEQB="31 29 31 30 31 30 31 31 30 31 30"
for j in $(seq 0 18); do
	y="$(( ($j + 1) * 126))"
	for x in 0 365 731 1096; do
		A="$A line $(($x + $BORDER)),$(($y + $BORDER - 16)) $(($x + $BORDER)),$(($y + $BORDER - 2))"
		A="$A text $(($x + $BORDER + 2)),$(($y + $BORDER + 2)) '${YEAR}'"
		if [ $(($YEAR % 4)) -eq 0 ]; then
			YMSEQ="$YMSEQB"
		else
			YMSEQ="$YMSEQA"
		fi
		moffs=0
		for m in $YMSEQ; do
			moffs=$(($moffs + $m))
			A="$A line $(($x + $BORDER + $moffs)),$(($y + $BORDER - 16)) $(($x + $BORDER + $moffs)),$(($y + $BORDER - 13))"
			donedays=$(($donedays + $m))
			if [ $donedays -ge $NDAYS ]; then
				break;
			fi
		done
		donedays=$(($donedays + 31))	# December
		YEAR="$(($YEAR + 1))"
		if [ $YEAR -gt $LASTYEAR ]; then
			break;
		fi
	done
	if [ $YEAR -gt $LASTYEAR ]; then
		break;
	fi
done

LASTDAY="$(date -d "31-dec-1948 + $NDAYS days" '+%Y%m%d')"
LASTYEARPIX="$((($NDAYS % (4*365+1)) + $BORDER))"

convert "$DIR/border.png" -stroke None -fill white \
	-font DejaVu-Sans-Bold -pointsize 18 \
	-draw "$A" \
	-font DejaVu-Sans -pointsize 16 \
	-annotate +950+2300 'NOAA NCEP/NCAR 2m temperature reanalysis.' \
	-annotate +950+2320 "Source data 19480101-$LASTDAY" \
	-annotate +950+2340 "Daily °C difference from average [${AVGSTARTYEAR}..$(($AVGSTARTYEAR + 30 - 1))]" \
	-annotate +950+2360 '365 day moving average and daily latitude average.' \
	-font DejaVu-Sans -pointsize 10 \
	-annotate +950+2377 'https://psl.noaa.gov/data/gridded/data.ncep.reanalysis.html' \
	-annotate +950+2391 'Reanalysis data provided by NOAA/OAR/ESRL PSL Boulder Colorado USA. Dataset source reference:' \
	-font DejaVu-Sans-Oblique -pointsize 10 \
	-annotate +950+2405 'Kalnay et al., The NCEP/NCAR 40-year reanalysis project, Bull. Amer. Meteor. Soc., 77, 437-470, 1996' \
	-draw "image over 450,2330 0,0 '$DIR/scale.png'" \
	-font DejaVu-Sans -pointsize 12 -stroke None \
	-draw "line $(($LASTYEARPIX + 3)),2294 $(($LASTYEARPIX + 42)),2294" \
	-draw "line $(($LASTYEARPIX + 3)),2387 $(($LASTYEARPIX + 42)),2387" \
	-annotate +$(($LASTYEARPIX + 45))+2304 'North-pole' \
	-annotate +$(($LASTYEARPIX + 45))+2345 '- equator -' \
	-annotate +$(($LASTYEARPIX + 45))+2386 'South-pole' \
	-draw "translate 890,2335 line 0,4 4,14 line 4,14 6,0 line 6,0 22,0 line 23,0 24,3 text 8,12 '13'" \
	-font Liberation-Sans-Narrow-Bold -pointsize 12 \
	-annotate +870+2363 "($(date '+%Y-%b-%d'))" \
	-annotate +590+2324 "(±0.1 °C mapped to white)" \
	-annotate +567+2397 "(data range min/max: $MINMAX)" \
	"$DIR/${IMGBASENAME}.png"
convert "$DIR/${IMGBASENAME}.png" -scale '512' "$DIR/${IMGBASENAME}-small.png"
exiv2 -M"set Exif.Image.Copyright Copyright, Vagrearg, $(date '+%Y'). CC-BY-SA 3.0" "$DIR/${IMGBASENAME}.png"
exiv2 -M"set Exif.Image.Copyright Copyright, Vagrearg, $(date '+%Y'). CC-BY-SA 3.0" "$DIR/${IMGBASENAME}-small.png"

rm "$DIR/part_"*.png "$DIR/mont.png" "$DIR/border.png" "$DIR/minmax.txt" "$DIR/lines.ppm"
