#include <getopt.h>
#include <QGuiApplication>
#include <QImage>
#include <QPainter>
#include <QDate>
#include <QThreadPool>
#include <QRunnable>
#include <QCommandLineParser>

QString sdir(".");
QString ddir(".");

#define V13X	736
#define V13Y	390
static QImage base;
static QImage world;
static const QPointF V13[5] = {
	QPointF(V13X+ 0, V13Y+ 4),
	QPointF(V13X+ 4, V13Y+14),
	QPointF(V13X+ 6, V13Y+ 0),
	QPointF(V13X+22, V13Y+ 0),
	QPointF(V13X+23, V13Y+ 3),
};

static const QPointF V13half[5] = {
	QPointF((V13X+ 0)/2.0, (V13Y+ 4)/2.0),
	QPointF((V13X+ 4)/2.0, (V13Y+14)/2.0),
	QPointF((V13X+ 6)/2.0, (V13Y+ 0)/2.0),
	QPointF((V13X+22)/2.0, (V13Y+ 0)/2.0),
	QPointF((V13X+23)/2.0, (V13Y+ 3)/2.0),
};

static int nyavg = 30;
static int ystart = 1948;
static int ndays = 365;
static bool half = false;

void handleImage(int id, int ido)
{
	QDate date = QDate(1948, 1, 1).addDays(id + ndays);
	QString s = QString(sdir + "/%1.ppm").arg(id, 5, 10, QChar('0'));
	QImage img(s);
	if(img.isNull()) {
		fprintf(stderr, "fail read %s\n", s.toUtf8().constData());
		return;
	}
	QImage m(img.width()*4, img.height()*4 + base.height(), QImage::Format_RGB32);
	QRect scr(0, 0, img.width()*4, img.height()*4);
	QPainter p(&m);
	p.setRenderHint(QPainter::Antialiasing);
	p.setRenderHint(QPainter::TextAntialiasing);
	p.setRenderHint(QPainter::SmoothPixmapTransform);
	p.drawImage(scr, img, img.rect());
	p.drawImage(scr, world, world.rect());
	p.drawImage(QRect(0, img.height()*4, base.width(), base.height()), base, base.rect());
	p.setFont(QFont("Sans", 11));
	p.drawText(8, 396, QString("%1 day moving average ending").arg(ndays));
	p.setFont(QFont("Sans", 15));
	p.drawText(8, 422, date.toString("yyyy-MM-dd"));
	p.setFont(QFont("Sans", 6));
	p.drawText(140, 408, "Data from NOAA - NCEP");
	p.drawText(140, 418, "Reanalysis air 2m gauss");
	p.drawText(140, 428, QString("{day(x) - avg[%1...%2]}").arg(ystart).arg(ystart + nyavg - 1));
	p.setFont(QFont("Sans", 8));
	p.drawText(V13X+8, V13Y+12, "13");
	p.drawPolyline(V13, 5);
	p.end();
	QString d = QString(ddir + "/%1.ppm.png").arg(ido, 5, 10, QChar('0'));
	m.save(d);
}

void handleImageHalf(int id, int ido)
{
	QDate date = QDate(1948, 1, 1).addDays(id + ndays);
	QString s = QString(sdir + "/%1.ppm").arg(id, 5, 10, QChar('0'));
	QImage img(s);
	if(img.isNull()) {
		fprintf(stderr, "fail read %s\n", s.toUtf8().constData());
		return;
	}
	QImage m(img.width()*2, img.height()*2 + base.height(), QImage::Format_RGB32);
	QRect scr(0, 0, img.width()*2, img.height()*2);
	QPainter p(&m);
	p.setRenderHint(QPainter::Antialiasing);
	p.setRenderHint(QPainter::TextAntialiasing);
	p.setRenderHint(QPainter::SmoothPixmapTransform);
	p.drawImage(scr, img, img.rect());
	p.drawImage(scr, world, world.rect());
	p.drawImage(QRect(0, img.height()*4, base.width(), base.height()), base, base.rect());
	p.setFont(QFont("Sans", 5.5));
	p.drawText(4, 198, QString("%1 day moving average ending").arg(ndays));
	p.setFont(QFont("Sans", 7.5));
	p.drawText(4, 211, date.toString("yyyy-MM-dd"));
	p.setFont(QFont("Sans", 3));
	p.drawText(70, 204, "Data from NOAA - NCEP");
	p.drawText(70, 209, "Reanalysis air 2m gauss");
	p.drawText(70, 214, QString("{day(x) - avg[%1...%2]}").arg(ystart).arg(ystart + nyavg - 1));
	p.setFont(QFont("Sans", 4));
	p.drawText((V13X+8)/2.0, (V13Y+12)/2.0, "13");
	p.drawPolyline(V13half, 5);
	p.end();
	QString d = QString(ddir + "/%1.ppm.png").arg(ido, 5, 10, QChar('0'));
	m.save(d);
}
class mythread : public QRunnable
{
private:
	int _start;
	int _n;
	int _ofs;
public:

	mythread(int start, int n, int o)
		: _start(start), _n(n), _ofs(o)
	{}

	void run() override {
		if(half) {
			for(int i = _start; i < _start + _n; i++)
				handleImageHalf(i, i + _ofs);
		} else {
			for(int i = _start; i < _start + _n; i++)
				handleImage(i, i + _ofs);
		}
	}
};

static const char usage_str[] =
	"qannotate -s <sourcedir> -d <destdir>\n"
	"Options:\n"
	"  -h   This help message\n"
	"  -d <dir> Destination directory\n"
	"  -s <dir> Source directory\n"
	"  -n <int> Number of images\n"
	"  -f <int> First image number\n"
	"  -o <int> Output image number offset\n"
	"  -x <int> # of years in average (default: 30)\n"
	"  -y <int> Start year in average (default: 1948)\n"
	"  -z <int> # of days in moving average smoothing (default: 365)\n"
	;
int main(int argc, char **argv)
{
	QGuiApplication app(argc, argv);

	int optc;
	int lose = 0;
	int nimg = 1;
	int fimg = 0;
	int oimg = 0;
	char *eptr;

	while(EOF != (optc = getopt(argc, argv, "Hhd:s:f:n:o:y:x:z:"))) {
		switch(optc) {
		case 'd':
			ddir = optarg;
			break;
		case 's':
			sdir = optarg;
			break;
		case 'h':
			printf("%s", usage_str);
			lose++;
			break;
		case 'f':
			fimg = strtol(optarg, &eptr, 0);
			if(nimg < 0 || *eptr) {
				fprintf(stderr, "must have positive index in -f\n");
				lose++;
			}
			break;
		case 'n':
			nimg = strtol(optarg, &eptr, 0);
			if(nimg < 1 || *eptr) {
				fprintf(stderr, "must have images in -n\n");
				lose++;
			}
			break;
		case 'o':
			oimg = strtol(optarg, &eptr, 0);
			if(abs(oimg) > 90000 || *eptr) {
				fprintf(stderr, "offset must be less than |90000| in -o\n");
				lose++;
			}
			break;
		case 'x':
			nyavg = strtol(optarg, &eptr, 0);
			if(nyavg < 1 || nyavg > 100 || *eptr) {
				fprintf(stderr, "# of average years must be between 1 and 100 in -x\n");
				lose++;
			}
			break;
		case 'y':
			ystart = strtol(optarg, &eptr, 0);
			if(ystart < 1800 || ystart > 2100 || *eptr) {
				fprintf(stderr, "Start year must be between 1800 and 2100 (yeah, right...) in -y\n");
				lose++;
			}
			break;
		case 'z':
			ndays = strtol(optarg, &eptr, 0);
			if(ndays < 1 || ndays > 5000 || *eptr) {
				fprintf(stderr, "# of days in moving average must be between 1 and 5000 in -z\n");
				lose++;
			}
			break;
		case 'H':
			half = true;
			break;
		default:
			lose++;
			break;
		}
	}

	if(half) {
		base = QImage(":/images/scale-384.png");
		world = QImage(":/images/earth_equirectangular-384.png");
	} else {
		base = QImage(":/images/scale-768.png");
		world = QImage(":/images/earth_equirectangular-768.png");
	}

	if(base.isNull()) {
		fprintf(stderr, "missing base image\n");
		lose++;
	}
	if(world.isNull()) {
		fprintf(stderr, "missing world image\n");
		lose++;
	}

	if(lose)
		return 1;

	const int NTHR = 8;
	int nx = nimg / NTHR;
	int nn = 0;
	if(nx > 5) {
		for(int i = 0; i < NTHR-1; i++) {
			mythread *t = new mythread(fimg + i*nx, nx, oimg);
			nn += nx;
			QThreadPool::globalInstance()->start(t);
		}
	}
	mythread *t = new mythread(fimg + (NTHR-1)*nx, nimg - nn, oimg);
	QThreadPool::globalInstance()->start(t);
	QThreadPool::globalInstance()->waitForDone();;

	app.exit(0);

	return 0;
}
